@@ -38,7 +38,7 @@ import (
 	"github.com/open-policy-agent/opa/ast"
 	errorpkg "github.com/pkg/errors"
 	corev1 "k8s.io/api/core/v1"
-	apiextensionsv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
@@ -202,14 +202,15 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 	// Watch for changes to ConstraintTemplateStatus
 	err = c.Watch(
 		&source.Kind{Type: &statusv1beta1.ConstraintTemplatePodStatus{}},
-		&handler.EnqueueRequestsFromMapFunc{ToRequests: &constrainttemplatestatus.Mapper{}})
+		handler.EnqueueRequestsFromMapFunc(constrainttemplatestatus.PodStatusToConstraintTemplateMapper(true)),
+	)
 	if err != nil {
 		return err
 	}
 
 	// Watch for changes to Constraint CRDs
 	err = c.Watch(
-		&source.Kind{Type: &apiextensionsv1beta1.CustomResourceDefinition{}},
+		&source.Kind{Type: &apiextensionsv1.CustomResourceDefinition{}},
 		&handler.EnqueueRequestForOwner{
 			OwnerType:    &v1beta1.ConstraintTemplate{},
 			IsController: true,
@@ -244,7 +245,7 @@ type ReconcileConstraintTemplate struct {
 
 // Reconcile reads that state of the cluster for a ConstraintTemplate object and makes changes based on the state read
 // and what is in the ConstraintTemplate.Spec
-func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reconcile.Result, error) {
+func (r *ReconcileConstraintTemplate) Reconcile(ctx context.Context, request reconcile.Request) (reconcile.Result, error) {
 	log := log.WithValues("template_name", request.Name)
 	// Short-circuit if shutting down.
 	if r.cs != nil {
@@ -260,7 +261,7 @@ func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reco
 	// Fetch the ConstraintTemplate instance
 	deleted := false
 	ct := &v1beta1.ConstraintTemplate{}
-	err := r.Get(context.TODO(), request.NamespacedName, ct)
+	err := r.Get(ctx, request.NamespacedName, ct)
 	if err != nil {
 		if !errors.IsNotFound(err) {
 			return reconcile.Result{}, err
@@ -285,7 +286,7 @@ func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reco
 		ctRef := &templates.ConstraintTemplate{}
 		ctRef.SetNamespace(request.Namespace)
 		ctRef.SetName(request.Name)
-		ctUnversioned, err := r.opa.GetTemplate(context.TODO(), ctRef)
+		ctUnversioned, err := r.opa.GetTemplate(ctx, ctRef)
 		result := reconcile.Result{}
 		if err != nil {
 			log.Info("missing constraint template in OPA cache, no deletion necessary")
@@ -294,6 +295,7 @@ func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reco
 		} else {
 			result, err = r.handleDelete(ctUnversioned)
 			if err != nil {
+				log.Error(err, "deletion error")
 				logError(request.NamespacedName.Name)
 				r.metrics.registry.add(request.NamespacedName, metrics.ErrorStatus)
 				return reconcile.Result{}, err
@@ -314,16 +316,19 @@ func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reco
 	status.Status.TemplateUID = ct.GetUID()
 	status.Status.ObservedGeneration = ct.GetGeneration()
 	status.Status.Errors = nil
+
 	unversionedCT := &templates.ConstraintTemplate{}
 	if err := r.scheme.Convert(ct, unversionedCT, nil); err != nil {
-		r.metrics.registry.add(request.NamespacedName, metrics.ErrorStatus)
 		log.Error(err, "conversion error")
+		r.metrics.registry.add(request.NamespacedName, metrics.ErrorStatus)
 		logError(request.NamespacedName.Name)
 		return reconcile.Result{}, err
 	}
+
 	unversionedProposedCRD, err := r.opa.CreateCRD(context.Background(), unversionedCT)
 	if err != nil {
-		r.tracker.CancelTemplate(unversionedCT) // Don't track templates that failed compilation
+		log.Error(err, "CRD creation error")
+		r.tracker.TryCancelTemplate(unversionedCT) // Don't track templates that failed compilation
 		r.metrics.registry.add(request.NamespacedName, metrics.ErrorStatus)
 		var createErr *v1beta1.CreateCRDError
 		if parseErrs, ok := err.(ast.Errors); ok {
@@ -344,11 +349,11 @@ func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reco
 		return reconcile.Result{}, nil
 	}
 
-	proposedCRD := &apiextensionsv1beta1.CustomResourceDefinition{}
+	proposedCRD := &apiextensionsv1.CustomResourceDefinition{}
 	if err := r.scheme.Convert(unversionedProposedCRD, proposedCRD, nil); err != nil {
-		r.tracker.CancelTemplate(unversionedCT) // Don't track templates that failed compilation
+		log.Error(err, "CRD conversion error")
+		r.tracker.TryCancelTemplate(unversionedCT) // Don't track templates that failed compilation
 		r.metrics.registry.add(request.NamespacedName, metrics.ErrorStatus)
-		log.Error(err, "conversion error")
 		logError(request.NamespacedName.Name)
 		err := r.reportErrorOnCTStatus("conversion_error", "Could not convert from unversioned resource", status, err)
 		return reconcile.Result{}, err
@@ -358,8 +363,8 @@ func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reco
 	namespace := unversionedProposedCRD.GetNamespace()
 	// Check if the constraint CRD already exists
 	action := updatedAction
-	currentCRD := &apiextensionsv1beta1.CustomResourceDefinition{}
-	err = r.Get(context.TODO(), types.NamespacedName{Name: name, Namespace: namespace}, currentCRD)
+	currentCRD := &apiextensionsv1.CustomResourceDefinition{}
+	err = r.Get(ctx, types.NamespacedName{Name: name, Namespace: namespace}, currentCRD)
 	switch {
 	case err == nil:
 		break
@@ -369,6 +374,7 @@ func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reco
 		currentCRD = nil
 
 	default:
+		log.Error(err, "client.Get CRD error")
 		logError(request.NamespacedName.Name)
 		r.metrics.registry.add(request.NamespacedName, metrics.ErrorStatus)
 		return reconcile.Result{}, err
@@ -376,6 +382,7 @@ func (r *ReconcileConstraintTemplate) Reconcile(request reconcile.Request) (reco
 
 	result, err := r.handleUpdate(ct, unversionedCT, proposedCRD, currentCRD, status)
 	if err != nil {
+		log.Error(err, "update error")
 		logError(request.NamespacedName.Name)
 		r.metrics.registry.add(request.NamespacedName, metrics.ErrorStatus)
 	} else if !result.Requeue {
@@ -401,7 +408,7 @@ func (r *ReconcileConstraintTemplate) reportErrorOnCTStatus(code, message string
 func (r *ReconcileConstraintTemplate) handleUpdate(
 	ct *v1beta1.ConstraintTemplate,
 	unversionedCT *templates.ConstraintTemplate,
-	proposedCRD, currentCRD *apiextensionsv1beta1.CustomResourceDefinition,
+	proposedCRD, currentCRD *apiextensionsv1.CustomResourceDefinition,
 	status *statusv1beta1.ConstraintTemplatePodStatus,
 ) (reconcile.Result, error) {
 	name := proposedCRD.GetName()
@@ -418,7 +425,7 @@ func (r *ReconcileConstraintTemplate) handleUpdate(
 			log.Error(err, "failed to report constraint template ingestion duration")
 		}
 		err := r.reportErrorOnCTStatus("ingest_error", "Could not ingest Rego", status, err)
-		r.tracker.CancelTemplate(unversionedCT) // Don't track templates that failed compilation
+		r.tracker.TryCancelTemplate(unversionedCT) // Don't track templates that failed compilation
 		return reconcile.Result{}, err
 	}
 
@@ -431,7 +438,7 @@ func (r *ReconcileConstraintTemplate) handleUpdate(
 	t.Observe(unversionedCT)
 	log.Info("[readiness] observed ConstraintTemplate", "name", unversionedCT.GetName())
 
-	var newCRD *apiextensionsv1beta1.CustomResourceDefinition
+	var newCRD *apiextensionsv1.CustomResourceDefinition
 	if currentCRD == nil {
 		newCRD = proposedCRD.DeepCopy()
 	} else {
@@ -514,8 +521,8 @@ func (r *ReconcileConstraintTemplate) deleteAllStatus(ctName string) error {
 	})); err != nil {
 		return err
 	}
-	for _, s := range cstrStatusObjs.Items {
-		if err := r.Delete(context.TODO(), &s); err != nil {
+	for index := range cstrStatusObjs.Items {
+		if err := r.Delete(context.TODO(), &cstrStatusObjs.Items[index]); err != nil {
 			if !errors.IsNotFound(err) {
 				return err
 			}
@@ -557,20 +564,14 @@ func (r *ReconcileConstraintTemplate) addWatch(kind schema.GroupVersionKind) err
 	if err := r.watcher.AddWatch(kind); err != nil {
 		return err
 	}
-	if err := r.statusWatcher.AddWatch(kind); err != nil {
-		return err
-	}
-	return nil
+	return r.statusWatcher.AddWatch(kind)
 }
 
 func (r *ReconcileConstraintTemplate) removeWatch(kind schema.GroupVersionKind) error {
 	if err := r.watcher.RemoveWatch(kind); err != nil {
 		return err
 	}
-	if err := r.statusWatcher.RemoveWatch(kind); err != nil {
-		return err
-	}
-	return nil
+	return r.statusWatcher.RemoveWatch(kind)
 }
 
 type action string
