@@ -39,9 +39,9 @@ import (
 	"github.com/open-policy-agent/gatekeeper/third_party/sigs.k8s.io/controller-runtime/pkg/dynamiccache"
 	"github.com/prometheus/client_golang/prometheus"
 	"golang.org/x/net/context"
-	admissionv1beta1 "k8s.io/api/admission/v1beta1"
+	admissionv1 "k8s.io/api/admission/v1"
 	corev1 "k8s.io/api/core/v1"
-	apiextensionsv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	errors2 "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -122,16 +122,15 @@ violation[{"msg": "denied!"}] {
 	}
 
 	// Uncommenting the below enables logging of K8s internals like watch.
-	//fs := flag.NewFlagSet("", flag.PanicOnError)
-	//klog.InitFlags(fs)
-	//fs.Parse([]string{"--alsologtostderr", "-v=10"})
-	//klog.SetOutput(os.Stderr)
+	// fs := flag.NewFlagSet("", flag.PanicOnError)
+	// klog.InitFlags(fs)
+	// fs.Parse([]string{"--alsologtostderr", "-v=10"})
+	// klog.SetOutput(os.Stderr)
 
 	// Setup the Manager and Controller.  Wrap the Controller Reconcile function so it writes each request to a
 	// channel when it is finished.
 	mgr, wm := setupManager(t)
-	c := &testclient.RetryClient{Client: mgr.GetClient()}
-	ctx := context.Background()
+	c := testclient.NewRetryClient(mgr.GetClient())
 
 	// creating the gatekeeper-system namespace is necessary because that's where
 	// status resources live by default
@@ -142,7 +141,6 @@ violation[{"msg": "denied!"}] {
 	backend, err := opa.NewBackend(opa.Driver(driver))
 	if err != nil {
 		t.Fatalf("unable to set up OPA backend: %s", err)
-
 	}
 	opa, err := backend.NewClient(opa.Targets(&target.K8sValidationTarget{}))
 	if err != nil {
@@ -152,7 +150,7 @@ violation[{"msg": "denied!"}] {
 	os.Setenv("POD_NAME", "no-pod")
 	podstatus.DisablePodOwnership()
 	cs := watch.NewSwitch()
-	tracker, err := readiness.SetupTracker(mgr)
+	tracker, err := readiness.SetupTracker(mgr, false)
 	g.Expect(err).NotTo(gomega.HaveOccurred())
 	pod := &corev1.Pod{}
 	pod.Name = "no-pod"
@@ -163,11 +161,12 @@ violation[{"msg": "denied!"}] {
 
 	cstr := newDenyAllCstr()
 
-	stopMgr, mgrStopped := StartTestManager(mgr, g)
+	ctx, cancelFunc := context.WithCancel(context.Background())
+	mgrStopped := StartTestManager(ctx, mgr, g)
 	once := sync.Once{}
 	testMgrStopped := func() {
 		once.Do(func() {
-			close(stopMgr)
+			cancelFunc()
 			mgrStopped.Wait()
 		})
 	}
@@ -175,7 +174,7 @@ violation[{"msg": "denied!"}] {
 	defer testMgrStopped()
 	// Clean up to remove the crd, constraint and constraint template
 	defer func() {
-		crd := &apiextensionsv1beta1.CustomResourceDefinition{}
+		crd := &apiextensionsv1.CustomResourceDefinition{}
 		g.Expect(c.Get(ctx, crdKey, crd)).NotTo(gomega.HaveOccurred())
 
 		g.Expect(deleteObject(ctx, c, cstr, timeout)).To(gomega.BeNil())
@@ -190,7 +189,7 @@ violation[{"msg": "denied!"}] {
 
 		clientset := kubernetes.NewForConfigOrDie(cfg)
 		g.Eventually(func() error {
-			crd := &apiextensionsv1beta1.CustomResourceDefinition{}
+			crd := &apiextensionsv1.CustomResourceDefinition{}
 			if err := c.Get(ctx, crdKey, crd); err != nil {
 				return err
 			}
@@ -221,7 +220,7 @@ violation[{"msg": "denied!"}] {
 				Name: "testns",
 			},
 		}
-		req := admissionv1beta1.AdmissionRequest{
+		req := admissionv1.AdmissionRequest{
 			Kind: metav1.GroupVersionKind{
 				Group:   "",
 				Version: "v1",
@@ -242,14 +241,14 @@ violation[{"msg": "denied!"}] {
 
 	log.Info("Running test: Deleted constraint CRDs are recreated")
 	t.Run("Deleted constraint CRDs are recreated", func(t *testing.T) {
-		crd := &apiextensionsv1beta1.CustomResourceDefinition{}
+		crd := &apiextensionsv1.CustomResourceDefinition{}
 		g.Expect(c.Get(ctx, crdKey, crd)).NotTo(gomega.HaveOccurred())
 		origUID := crd.GetUID()
-		crd.Spec = apiextensionsv1beta1.CustomResourceDefinitionSpec{}
+		crd.Spec = apiextensionsv1.CustomResourceDefinitionSpec{}
 		g.Expect(c.Delete(ctx, crd)).NotTo(gomega.HaveOccurred())
 
 		g.Eventually(func() error {
-			crd := &apiextensionsv1beta1.CustomResourceDefinition{}
+			crd := &apiextensionsv1.CustomResourceDefinition{}
 			if err := c.Get(ctx, crdKey, crd); err != nil {
 				return err
 			}
@@ -260,7 +259,7 @@ violation[{"msg": "denied!"}] {
 				return errors.New("Not yet deleted")
 			}
 			for _, cond := range crd.Status.Conditions {
-				if cond.Type == apiextensionsv1beta1.Established && cond.Status == apiextensionsv1beta1.ConditionTrue {
+				if cond.Type == apiextensionsv1.Established && cond.Status == apiextensionsv1.ConditionTrue {
 					return nil
 				}
 			}
@@ -300,13 +299,13 @@ violation[{"msg": "denied!"}] {
 					{
 						Target: "admission.k8s.gatekeeper.sh",
 						Rego: `
-package foo
+	package foo
 
-violation[{"msg": "hi"}] { 1 == 1 }
+	violation[{"msg": "hi"}] { 1 == 1 }
 
-anyrule[}}}//invalid//rego
+	anyrule[}}}//invalid//rego
 
-`},
+	`},
 				},
 			},
 		}
@@ -324,8 +323,8 @@ anyrule[}}}//invalid//rego
 				return err
 			}
 			if ct.Name == "invalidrego" {
-				status := getCTByPodStatus(ct)
-				if status == nil {
+				status, found := getCTByPodStatus(ct)
+				if !found {
 					return fmt.Errorf("could not retrieve CT status for pod, byPod status: %+v", ct.Status.ByPod)
 				}
 				if len(status.Errors) == 0 {
@@ -352,7 +351,7 @@ anyrule[}}}//invalid//rego
 				Name: "testns",
 			},
 		}
-		req := admissionv1beta1.AdmissionRequest{
+		req := admissionv1.AdmissionRequest{
 			Kind: metav1.GroupVersionKind{
 				Group:   "",
 				Version: "v1",
@@ -384,7 +383,7 @@ anyrule[}}}//invalid//rego
 	})
 }
 
-// Tests that expectations for constraints are cancelled if the corresponding constraint is deleted.
+// Tests that expectations for constraints are canceled if the corresponding constraint is deleted.
 func TestReconcile_DeleteConstraintResources(t *testing.T) {
 	log.Info("Running test: Cancel the expectations when constraint gets deleted")
 
@@ -392,8 +391,19 @@ func TestReconcile_DeleteConstraintResources(t *testing.T) {
 
 	// Setup the Manager
 	mgr, wm := setupManager(t)
-	c := &testclient.RetryClient{Client: mgr.GetClient()}
-	ctx := context.Background()
+	c := testclient.NewRetryClient(mgr.GetClient())
+	ctx, cancelFunc := context.WithCancel(context.Background())
+
+	// start manager that will start tracker and controller
+	mgrStopped := StartTestManager(ctx, mgr, g)
+	once := sync.Once{}
+	testMgrStopped := func() {
+		once.Do(func() {
+			cancelFunc()
+			mgrStopped.Wait()
+		})
+	}
+	defer testMgrStopped()
 
 	// Create the constraint template object and expect the Reconcile to be created when controller starts
 	instance := &v1beta1.ConstraintTemplate{
@@ -419,7 +429,8 @@ violation[{"msg": "denied!"}] {
 			},
 		},
 	}
-	err := c.Create(context.TODO(), instance)
+
+	err := c.Create(ctx, instance)
 	g.Expect(err).NotTo(gomega.HaveOccurred())
 
 	defer func() { g.Expect(ignoreNotFound(c.Delete(ctx, instance))).To(gomega.BeNil()) }()
@@ -437,7 +448,7 @@ violation[{"msg": "denied!"}] {
 
 	// Create the constraint for constraint template
 	cstr := newDenyAllCstr()
-	err = c.Create(context.Background(), cstr)
+	err = c.Create(ctx, cstr)
 	g.Expect(err).NotTo(gomega.HaveOccurred())
 
 	// creating the gatekeeper-system namespace is necessary because that's where
@@ -445,7 +456,7 @@ violation[{"msg": "denied!"}] {
 	g.Expect(createGatekeeperNamespace(mgr.GetConfig())).NotTo(gomega.HaveOccurred())
 
 	// Set up tracker
-	tracker, err := readiness.SetupTracker(mgr)
+	tracker, err := readiness.SetupTracker(mgr, false)
 	g.Expect(err).NotTo(gomega.HaveOccurred())
 
 	// initialize OPA
@@ -470,18 +481,6 @@ violation[{"msg": "denied!"}] {
 	rec, _ := newReconciler(mgr, opa, wm, cs, tracker, events, nil, func() (*corev1.Pod, error) { return pod, nil })
 	g.Expect(add(mgr, rec)).NotTo(gomega.HaveOccurred())
 
-	// start manager that will start tracker and controller
-	stopMgr, mgrStopped := StartTestManager(mgr, g)
-
-	once := sync.Once{}
-	testMgrStopped := func() {
-		once.Do(func() {
-			close(stopMgr)
-			mgrStopped.Wait()
-		})
-	}
-	defer testMgrStopped()
-
 	// get the object tracker for the constraint
 	ot := tracker.For(gvk)
 	tr, ok := ot.(testExpectations)
@@ -495,11 +494,10 @@ violation[{"msg": "denied!"}] {
 
 	// Delete the constraint , the delete event will be reconciled by controller
 	// to cancel the expectation set for it by tracker
-	g.Expect(c.Delete(context.TODO(), cstr)).NotTo(gomega.HaveOccurred())
+	g.Expect(c.Delete(ctx, cstr)).NotTo(gomega.HaveOccurred())
 
 	// set event channel to receive request for constraint
 	events <- event.GenericEvent{
-		Meta:   cstr,
 		Object: cstr,
 	}
 
@@ -540,16 +538,14 @@ func newDenyAllCstr() *unstructured.Unstructured {
 	return cstr
 }
 
-func getCTByPodStatus(templ *v1beta1.ConstraintTemplate) *v1beta1.ByPodStatus {
+func getCTByPodStatus(templ *v1beta1.ConstraintTemplate) (v1beta1.ByPodStatus, bool) {
 	statuses := templ.Status.ByPod
-	var status *v1beta1.ByPodStatus
 	for _, s := range statuses {
 		if s.ID == util.GetID() {
-			status = s
-			break
+			return s, true
 		}
 	}
-	return status
+	return v1beta1.ByPodStatus{}, false
 }
 
 func getCByPodStatus(obj *unstructured.Unstructured) (*statusv1beta1.ConstraintPodStatusStatus, error) {
@@ -582,32 +578,42 @@ func getCByPodStatus(obj *unstructured.Unstructured) (*statusv1beta1.ConstraintP
 }
 
 // makeCRD generates a CRD specified by GVK and plural for testing.
-func makeCRD(gvk schema.GroupVersionKind, plural string) *apiextensionsv1beta1.CustomResourceDefinition {
-	return &apiextensionsv1beta1.CustomResourceDefinition{
+func makeCRD(gvk schema.GroupVersionKind, plural string) *apiextensionsv1.CustomResourceDefinition {
+	trueBool := true
+	return &apiextensionsv1.CustomResourceDefinition{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: fmt.Sprintf("%s.%s", plural, gvk.Group),
 		},
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "CustomResourceDefinition",
-			APIVersion: "apiextensions/v1beta1",
+			APIVersion: "apiextensions/v1",
 		},
-		Spec: apiextensionsv1beta1.CustomResourceDefinitionSpec{
+		Spec: apiextensionsv1.CustomResourceDefinitionSpec{
 			Group: gvk.Group,
-			Names: apiextensionsv1beta1.CustomResourceDefinitionNames{
+			Names: apiextensionsv1.CustomResourceDefinitionNames{
 				Plural:   plural,
 				Singular: strings.ToLower(gvk.Kind),
 				Kind:     gvk.Kind,
 			},
-			Versions: []apiextensionsv1beta1.CustomResourceDefinitionVersion{
-				{Name: gvk.Version, Served: true, Storage: true},
+			Versions: []apiextensionsv1.CustomResourceDefinitionVersion{
+				{
+					Name:    gvk.Version,
+					Served:  true,
+					Storage: true,
+					Schema: &apiextensionsv1.CustomResourceValidation{
+						OpenAPIV3Schema: &apiextensionsv1.JSONSchemaProps{
+							XPreserveUnknownFields: &trueBool,
+						},
+					},
+				},
 			},
-			Scope: apiextensionsv1beta1.ClusterScoped,
+			Scope: apiextensionsv1.ClusterScoped,
 		},
 	}
 }
 
 // applyCRD applies a CRD and waits for it to register successfully.
-func applyCRD(ctx context.Context, g *gomega.GomegaWithT, client client.Client, gvk schema.GroupVersionKind, crd runtime.Object) error {
+func applyCRD(ctx context.Context, g *gomega.GomegaWithT, client client.Client, gvk schema.GroupVersionKind, crd client.Object) error {
 	err := client.Create(ctx, crd)
 	if err != nil {
 		return err
@@ -624,7 +630,7 @@ func applyCRD(ctx context.Context, g *gomega.GomegaWithT, client client.Client,
 	return nil
 }
 
-func deleteObject(ctx context.Context, c client.Client, obj runtime.Object, timeout time.Duration) error {
+func deleteObject(ctx context.Context, c client.Client, obj client.Object, timeout time.Duration) error {
 	err := c.Delete(ctx, obj)
 	if err != nil {
 		if errors2.IsNotFound(err) {
@@ -635,7 +641,7 @@ func deleteObject(ctx context.Context, c client.Client, obj runtime.Object, time
 
 	err = wait.Poll(time.Second, timeout, func() (bool, error) {
 		// Get the object name
-		name, _ := client.ObjectKeyFromObject(obj)
+		name := client.ObjectKeyFromObject(obj)
 		err := c.Get(ctx, name, obj)
 		if err != nil {
 			if errors2.IsGone(err) || errors2.IsNotFound(err) {
